from datetime import datetime
from sqlalchemy import Column, Integer, String, DateTime, ForeignKey, Text
from sqlalchemy.orm import relationship
from models.base import Base, Session
from models.enums import MediaType, DownloadStatus

class MediaDownload(Base):
    __tablename__ = 'media_downloads'
    
    id = Column(Integer, primary_key=True)
    message_id = Column(Integer, ForeignKey('messages.id'))
    media_type = Column(String)
    download_status = Column(String)
    download_timestamp = Column(DateTime, default=datetime.utcnow)
    retry_count = Column(Integer, default=0)
    error_message = Column(Text, nullable=True)
    local_url = Column(String, nullable=True)
    message = relationship("Message", back_populates="media_downloads")

    @classmethod
    def record_download_attempt(cls, message_id, media_type, status, local_url=None, error_message=None):
        """
        Record a download attempt with optional local_url and error_message
        """
        try:
            # Convert string to enum if it's a string
            if isinstance(media_type, str):
                media_type = MediaType(media_type.lower())
            if isinstance(status, str):
                status = DownloadStatus(status.lower())

            session = Session()
            download = cls(
                message_id=message_id,
                media_type=media_type.value,
                download_status=status.value,
                local_url=local_url,
                error_message=error_message
            )
            session.add(download)
            session.commit()
            return download
        except Exception as e:
            if session:
                session.rollback()
            raise e
        finally:
            if session:
                session.close()
